"""\
    This is an Xchat2 frontend for Dashboard
    When loaded it automatically starts sending cluepackets containing the irc_name
    of chatters, and a text_block of what they said
    
    To turn cluepacket sending off use  "/cluepackets off"
    them to turn them back on again use "/cluepackets on"
    
    To turn on debugging (not very helpful at the moment) use "/cluedebug <level>"
    Debugging currently supports the levels 'On', 'Off', and 'Verbose'
    
    There are two forms of cluepackets generated by this plugin, internally referred to
    as 'focused' and 'refocus'. 'Focused' packets are generated when text is generated
    in the focusecd context (where context can generally be thought of as Tab), 'Refocus'
    packets are generated when you switch to a context, this sends the last x number of
    lines from the newly focused context (x defaults to 20, but can be changed by the
    user through the use of the "/ClueMemory <int>" command
    
    The included Dashboard class is taken from the dashboard.py file
    originally found at:
    http://cvs.gnome.org/lxr/source/dashboard/frontends/dashboard.py
"""

class Dashboard:
    def __init__(self, frontend='dashboard.py', host='localhost',
                 port=5913, debug=0, context='NoneAssigned'):
        self.frontend = frontend
        self.host     = host
        self.port     = port
        self.debug    = debug

    def _connect(self):
        import socket, errno
        sock = socket.socket(socket.AF_INET, socket.SOCK_STREAM)
        sock.setblocking(0)

        try:
            sock.connect((self.host, self.port))
        except socket.error, e:
            if e.args[0] == errno.EINPROGRESS:
                return sock
            else:
                raise

    def _escape(self, data):
        data = data.replace("&", "&amp;")
        data = data.replace("<", "&lt;")
        data = data.replace(">", "&gt;")
        return data

    def send_cluepacket(self, context, focused, clues):
        try:
            sock = self._connect()
    
            cluestrs = ['<CluePacket>',
                        '<Frontend>%s</Frontend>' % self._escape(self.frontend),
                        '<Context>%s</Context>' % self._escape(context)
                        ]

            if focused:
                cluestrs.append('<Focused>True</Focused>')
            else:
                cluestrs.append('<Focused>False</Focused>')
            
            for clue in clues:
                attrs = ''
                for key in ['type', 'relevance']:
                    if clue.has_key(key):
                        attrs += ' %s="%s"' % \
                                 (key.capitalize(),
                                  self._escape(clue[key].__str__()))
    
                cluestrs.append('<Clue%s>%s</Clue>' \
                                % (attrs, self._escape(clue['data'])))
    
            cluestrs.append('</CluePacket>\n')
    
            import string
            str = string.join(cluestrs, '')

            sock.send(str)
            sock.close()
        except:
            if self.debug:
                raise
                
### XChat Plugin for Dashboard

import xchat
import os
import thread
import sys

LINES_PER_CONTEXT = 20

focused_nick_relevance = 10          #The relevance value of nicks sent when the context they are generated from are focused
refocus_nick_relevance = 6           #The relevance value of nicks sent when a context gains focus

focused_text_relevance = 7         #The relevance value of text sent when the context they are generated from are focused
refocus_text_relevance = 3         #The relevance value of text sent when a context gains focus

send_clues  = 1
debug = 0

xchat_directory = xchat.get_info("xchatdir")+ os.sep    #users xchat directory (for storing preferences)
preferences_file_name = "dashboard.conf"                #filename to store the users preferences in

current_context_id = None       #ID of the context being worked on
focused_context_id = None       #ID of focused context
old_context_id = None           #name of old context
contexts = {}                   #dictionary of contexts, key is the name of the context as found by get_context_id(context)
                                #contents is a list of Dictionarys, these Dictionarys contains 2 keys, Nick, and Sentence.
frontend = "Xchat2"
dash = Dashboard(frontend)

__module_name__        = "XChat2 Cluepacket Generator"
__module_version__     = "0.8"
__module_description__ = "Generates cluepackets for other cluepacket aware programs from xchat"

print("Cluepacket Plugin Loaded")

def format_nick(unformatted_nick):
    formatted_nick = unformatted_nick
    if unformatted_nick[0] == "": # if the first character is the colour control character
        formatted_nick = unformatted_nick[1:] # remove it
        while formatted_nick[0].isdigit(): #and strip off all subsequent numerics
            formatted_nick = formatted_nick[1:len(unformatted_nick)]
    return formatted_nick

def get_context_id(myContext):
    '''Returns a string to uniquely identify Contexts'''
    try:
        return myContext.get_info("server") + "||" + myContext.get_info("channel")
    except TypeError:
        #caused by myContext not having a server or channel, due to connection problems
        return "Default"
        
def delete_context_by_id(context_id):
    ''' Dereference the context who's id matches context_id'''
    try:
        del contexts[context_id]
    except KeyError:
        pass
        #Key didn't exist in contexts
        
def find_context_by_id(context_id):
    '''Examines the contexts dictionary to see if the specified context ID exists as a key in it'''
    import __main__
    try:
        t = contexts[context_id]
    except KeyError:
        return 0
    return 1

def force_add_context(context):
    '''Adds a context to the dictionary'''
    import __main__        
    __main__.contexts[get_context_id(context)] = []
    
def add_context(context):
    '''Adds a context to the dictionary, as long as the key does not already exist in it'''
    if find_context_by_id(get_context_id(context)) == 0:
        force_add_context(context)

def add_line_to_context(context_id, nick, said):
    '''Adds a "Spoken" dictionary to the list
    The dictionary has two keys, 'Nick' and 'Sentence'
    The Nick is who spoke a phrase, and Sentence is what they said'''
    import __main__
    contexts[context_id].insert(1,{'Nick':nick, 'Sentence':said})
    try:
        contexts[context_id].pop(20)
    except:
        pass
        #Not 20 entrys yet!
        
def new_context(word, word_eol, userdata):
    '''Creates a new context, called by xchat'''
    import __main__
    add_context(xchat.get_context())

def focus_tab(word, word_eol, userdata):
    '''Called when xchat changes tab'''
    import __main__
    __main__.old_context_id = __main__.focused_context_id

    if find_context_by_id(get_context_id(xchat.get_context())) == 0:
        add_context(xchat.get_context())          
        
    __main__.focused_context_id = get_context_id(xchat.get_context())

    cluepacket = []
    dict = None
    for dict in contexts[focused_context_id]:
        try:
            cluepacket.append({'data' : dict['Nick'], 'type' : 'irc_name', 'relevance' : refocus_nick_relevance})
        except KeyError:
            pass
            #caused by messages not associated with nicks... channel messages?
        try:
            cluepacket.append({'data' : dict['Sentence'], 'type' : 'textblock', 'relevance' : refocus_text_relevance })
        except KeyError:
            pass
            #caused by messages that dont have sentences... (not sure how, but they seem to happen)
    dash.send_cluepacket(focused_context_id, 1, cluepacket)
        
def text_cb(word, word_eol, userdata):
    '''called by xchat whenever a message is sent by a user'''
    import __main__
    
    add_context(xchat.get_context())                            #ensure the context exists for storing old message purposes
    current_context_id = get_context_id(xchat.get_context())
    
    
    sentence = ""
    individual_word = ""
    try:
        for individual_word in word[1]:
            sentence = sentence + individual_word
    except IndexError:
        pass # Nothing was said...
      
    nick = format_nick(word[0])
  
    if current_context_id == focused_context_id:
        cluepacket = [{'data' : nick , 'type' : 'irc_name', 'relevance' : focused_nick_relevance} , {'type' : 'textblock', 'data' : sentence, 'relevance': focused_text_relevance}]
        dash.send_cluepacket(current_context_id, 1, cluepacket)
    
    add_line_to_context(get_context_id(xchat.get_context()), nick, sentence)
    
    return xchat.EAT_NONE

#
# Command Handlers
#
    
def dump_context_info(word, word_eol, userdata):
    import __main__
    i =0 
    for text in contexts[get_context_id(xchat.get_context())]:
        i = i + 1
    print i
    return xchat.EAT_ALL
        
def delete_context(word, word_eol, userdata):
    delete_context_by_id(get_context_id(xchat.get_context()))
    
def start_handlers():
    ''' hooks into the text handling of xchat'''
    xchat.hook_print("Open Context", new_context)
    xchat.hook_print("Focus Tab", focus_tab)
    xchat.hook_print("Close Context", delete_context)
    
    xchat.hook_print("Channel Message",  text_cb)
    xchat.hook_print("Channel Message Hilight", text_cb)
    xchat.hook_print("Private Message", text_cb)
    xchat.hook_print("Private Message to Dialog", text_cb)
    xchat.hook_print("Message Send", text_cb)
    xchat.hook_print("Your Message", text_cb)
    xchat.hook_print("Channel Action", text_cb)
    xchat.hook_print("Channel Action Hilight", text_cb)

def stop_handlers():
    ''' Unhooks the text handling '''
    xchat.unhook(text_cb)
    xchat.unhook(new_context)
    xchat.unhook(focus_tab)

def cluepackets_command( word, word_eol, userdata ):
    '''allows you to turn cluepacket sending on and off'''
    import __main__

    if __main__.debug >= 2:
        print "cluepackets_command(word, word_eol, userdata)"

    if len(word) < 2:
        print "You did not supply enough args."
        return xchat.EAT_ALL
    if word[1] == "on":
        __main__.send_clues = 1
        print "Turning Cluepackets on."
        start_handlers()     
    elif word[1] == "off":
        __main__.send_clues = 0
        print "Turning Cluepackets off."
        stop_handlers()

    return xchat.EAT_ALL
    
def cluepackets_set_relevance(word, word_eol, userdata):
    import __main__
    
    if len(word) < 3:
        print "You did not supply enough args."
        return xchat.EAT_ALL
    
    if word[1] == "focused": # omitted a target type - apply relevance to both
        try:
            if 11 > int(word[2]) > -1:
                __main__.focused_text_relevance = word[2]
                __main__.focused_nick_relevance = word[2]
                print "Set focused text and nick relevance to " + word[2]
        except:
            print "Unable to set focused text and nick relevance to " + word[2]
    elif word[1] == "refocus": #omitted a target type - apply relevance to both
        try:
            if 11 > int(word[2]) > -1:
                __main__.refocus_text_relevance = word[2]
                __main__.refocus_nick_relevance = word[2]
                print "Set refocused text and nick relevance to " + word[2]
        except:
            print "Unable to set refocused text and nick relevance to " + word[2]
    elif word[1] == "text": #targeted text
        if word[2] == "focused":
            try:
                if 11 > int(word[3]) > -1:
                    __main__.focused_text_relevance = word[3]
                    print "Set focused text relevance to " + word[3]
            except:
                print "Unable to set focused text relevance to " + word[3]
        elif word[2] == "refocus":
            try:
                if 11 > int(word[3]) > -1:
                    __main__.refocus_text_relevance = word[3]
                    print "Set refocused text relevance to " + word[3]
            except:
                print "Unable to set refocused text relevance to " + word[3]
    elif word[1] == "nick": #targetted nicks
        if word[2] == "focused":
            try:
                if 11 > int(word[3]) > -1:
                    __main__.focused_nick_relevance = word[3]
                    print "Set focused nick relevance to " + word[3]
            except:
                print "Unable to set focused nick relevance to " + word[3]
        elif word[2] == "refocus":
            try:
                if 11 > int(word[3]) > -1:
                    __main__.refocus_nick_relevance = word[3]
                    print "Set refocused nick relevance to " + word[3]
            except:
                print "Unable to set refocused nick relevance to " + word[3]
    
    save_preferences()
    return xchat.EAT_ALL
    
def cluepackets_set_memory(word, word_eol, userdata):
    import __main__
    
    if len(word) < 2:
        print "You did not supply enough args."
        return xchat.EAT_ALL
    
    try:
        __main__.LINES_PER_CONTEXT = word[1]
    except:
        print "Unable to set cluepacket memory to " + word[1] + " lines"
    return xchat.EAT_ALL

def cluepackets_set_debug(word, word_eol, userdata ):
    ''' Most debugging support has not been put in yet!'''
    import __main__

    if __main__.debug >= 2:
        print "cluepackets_set_debug(word, word_eol, userdata)"

    if len(word) < 2:
        print "You did not supply enough args."
        return xchat.EAT_ALL
    if word[1] == "on":
        __main__.debug = 1
        print "Turning Cluepacket debugging on"
    elif word[1] == "off":
        __main__.debug = 0
        print "Turning Cluepacket debugging off"
    elif word[1] == "verbose":
        __main__.debug = 2
        print "Verbose Cluepacket debugging"
    return xchat.EAT_ALL

def save_preferences():
    focused_text_relevance_string = "focused_text_relevance = " + str(focused_text_relevance)
    focused_nick_relevance_string = "focused_nick_relevance = " + str(focused_nick_relevance)
    refocus_text_relevance_string = "refocus_text_relevance = " + str(refocus_text_relevance)
    refocus_nick_relevance_string = "refocus_nick_relevance = " + str(refocus_nick_relevance)
    context_memory_string         = "context_memory = " + str(LINES_PER_CONTEXT)
    try:
        preferences_file = open(xchat_directory + preferences_file_name, "w")
    
        preferences_file.write("//This file is automatically generated, and any non-standard variables in it will be removed when the plugin is loaded"  +os.linesep)
        preferences_file.write(focused_text_relevance_string + os.linesep)
        preferences_file.write(focused_nick_relevance_string + os.linesep)
        preferences_file.write(refocus_text_relevance_string + os.linesep)
        preferences_file.write(refocus_nick_relevance_string + os.linesep)
        preferences_file.write(context_memory_string + os.linesep)
    except:
        print "Unable to save preferences file"  

def load_preferences():
    import __main__

    value = None

    try:
        preferences_file = open(xchat_directory + preferences_file_name)

    
        for line in preferences_file:
            line = line.expandtabs(1)                       #convert tabs to spaces
            line = line.replace(" ","", line.count(" "))    #and strip all spaces... Voila! No more whitespace
            line = line.lower()                             #and now we have an easy to play with line!
            if   line.startswith("focused_text_relevance="):
                try:
                    value = int(line[line.find("=")+1:])
                    if -1 < value < 11:
                        __main__.focused_text_relevance = value
                    else:
                        print "Value for Focused Text Relevance is invalid"
                except:
                    print "Value for Focused Text Relevance is invalid"
            elif line.startswith("focused_nick_relevance="):
                try:
                    value = int(line[line.find("=")+1:])
                    if -1 < value < 11:
                        __main__.focused_nick_relevance = value
                    else:
                        print "Value for Focused nick Relevance is invalid"
                except:
                    print "Value for Focused nick Relevance is invalid"
            elif line.startswith("refocus_text_relevance="):
                try:
                    value = int(line[line.find("=")+1:])
                    if -1 < value < 11:
                        __main__.refocus_text_relevance = value
                    else:
                        print "Value for Refocus Text Relevance is invalid"
                except:
                    print "Value for Refocus Text Relevance is invalid"
            elif line.startswith("refocus_nick_relevance="):
                try:
                    value = int(line[line.find("=")+1:])
                    if -1 < value < 11:
                        __main__.refocus_nick_relevance = value
                    else:
                        print "Value for Refocus nick Relevance is invalid"
                except:
                    print "Value for Refocus nick Relevance is invalid"
            elif line.startswith("context_memory="):
                try:
                    value = int(line[line.find("=")+1:])
                    __main__.LINES_PER_CONTEXT= value
                except:
                    print "Value for the amount of lines to save per context is invalid"
                    
        preferences_file.close()
    except:
        print "Error reading preferences file"
        
    save_preferences()

xchat.hook_command( "CLUEPACKETS", cluepackets_command, help="/cluepackets <arg>, turn \"on\" or \"off\"." )
xchat.hook_command( "CLUERELEVANCE", cluepackets_set_relevance, help = "/ClueRelevance <nick|text (if omitted, applys to both)> <Focused|Unfocused> <(0-10)>" )
xchat.hook_command( "SET_CLUE_MEMORY", cluepackets_set_memory, help = "/ClueMemory x")
xchat.hook_command( "CLUEDEBUG", cluepackets_set_debug, help="/cluedebug <arg>, turn \"on\" or \"off\"." )
xchat.hook_command( "DUMP_CONTEXT_INFO", dump_context_info, help="/dump_context_info")

start_handlers()
load_preferences()
